# Copyright (c) 2012 Cloudera, Inc. All rights reserved.

import copy

import cmf.monitor.namenode.json_parser
import cmf.monitor.namenode.json_parser_jmx
import cmf.monitor.datanode.json_parser
import cmf.monitor.datanode.json_parser_jmx
import cmf.monitor.journalnode.json_parser_jmx
import cmf.monitor.secondarynamenode.json_parser
import cmf.monitor.secondarynamenode.json_parser_jmx
import cmf.monitor.master.json_parser
import cmf.monitor.regionserver.json_parser
import cmf.monitor.jobtracker.json_parser
import cmf.monitor.tasktracker.json_parser
import cmf.monitor.tasktracker.metrics
import cmf.monitor.servicemonitor.json_parser
import cmf.monitor.impalad.json_parser
import cmf.monitor.statestore.json_parser

# Refers to the JMX format exposed by Hadoop daemons
JMX_FORMAT = 'jmx'
# Refers to the 'metrics' format exposed by Hadoop daemons
METRICS_FORMAT = 'metrics'
# Refers to the format exposed by Impala daemons
IMPALA_FORMAT = 'impala'
# Refers to the JMX format exposed by the CM management roles.
MGMT_FORMAT = 'mgmt'

ROLE_CONTEXT = 'role'

ENTERPRISE = 'enterprise'
CDH3 = 'cdh3'
CDH4 = 'cdh4'

class MetricDetails:
    def __init__(self, format, version, context, sample):
        self.format = format
        self.version = version
        self.context = context
        self.sample = sample

# FLUME and ZOOKEEPER are handled specially below.
PLATFORM_METRICS = dict(
    # HDFS
    NAMENODE=\
        [MetricDetails(METRICS_FORMAT,
                       CDH3,
                       ROLE_CONTEXT,
                       cmf.monitor.namenode.json_parser.SAMPLE),
         MetricDetails(JMX_FORMAT,
                       CDH4,
                       ROLE_CONTEXT,
                       cmf.monitor.namenode.json_parser_jmx.SAMPLE)],
    SECONDARYNAMENODE=\
        [MetricDetails(METRICS_FORMAT,
                       CDH3,
                       ROLE_CONTEXT,
                       cmf.monitor.secondarynamenode.json_parser.SAMPLE),
         MetricDetails(JMX_FORMAT,
                       CDH4,
                       ROLE_CONTEXT,
                       cmf.monitor.secondarynamenode.json_parser_jmx.SAMPLE)],
    DATANODE=\
        [MetricDetails(METRICS_FORMAT,
                       CDH3,
                       ROLE_CONTEXT,
                       cmf.monitor.datanode.json_parser.SAMPLE),
         MetricDetails(JMX_FORMAT,
                       CDH4,
                       ROLE_CONTEXT,
                       cmf.monitor.datanode.json_parser_jmx.SAMPLE)],
    JOURNALNODE=\
        [MetricDetails(JMX_FORMAT,
                       CDH4,
                       ROLE_CONTEXT,
                       cmf.monitor.journalnode.json_parser_jmx.SAMPLE)],
    # MR
    JOBTRACKER=\
        [MetricDetails(METRICS_FORMAT,
                       CDH3,
                       ROLE_CONTEXT,
                       cmf.monitor.jobtracker.json_parser.SAMPLE_CDH3),
         MetricDetails(METRICS_FORMAT,
                       CDH4,
                       ROLE_CONTEXT,
                       cmf.monitor.jobtracker.json_parser.SAMPLE_CDH4)],
    TASKTRACKER=\
        [MetricDetails(METRICS_FORMAT,
                       CDH3,
                       ROLE_CONTEXT,
                       cmf.monitor.tasktracker.json_parser.SAMPLE_CDH3),
         MetricDetails(METRICS_FORMAT,
                       CDH4,
                       ROLE_CONTEXT,
                       cmf.monitor.tasktracker.json_parser.SAMPLE_CDH4)],

    # HBASE
    MASTER=\
        [MetricDetails(METRICS_FORMAT,
                       CDH3,
                       ROLE_CONTEXT,
                       cmf.monitor.master.json_parser.SAMPLE_CDH3),
         MetricDetails(METRICS_FORMAT,
                       CDH4,
                       ROLE_CONTEXT,
                       cmf.monitor.master.json_parser.SAMPLE_CDH4)],
    REGIONSERVER=\
        [MetricDetails(METRICS_FORMAT,
                       CDH3,
                       ROLE_CONTEXT,
                       cmf.monitor.regionserver.json_parser.SAMPLE_CDH3),
         MetricDetails(METRICS_FORMAT,
                       CDH4,
                       ROLE_CONTEXT,
                       cmf.monitor.regionserver.json_parser.SAMPLE_CDH4)],

    # IMPALA
    IMPALAD=\
        [MetricDetails(IMPALA_FORMAT,
                       CDH4,
                       ROLE_CONTEXT,
                       cmf.monitor.impalad.json_parser.SAMPLE_CDH4)],
    STATESTORE=\
        [MetricDetails(IMPALA_FORMAT,
                       CDH4,
                       ROLE_CONTEXT,
                       cmf.monitor.statestore.json_parser.SAMPLE_CDH4)],

    # MGMT - note that this list only contains management services
    # that expose metrics through a HTTP endpoint.
    HOSTMONITOR=\
        [MetricDetails(MGMT_FORMAT,
                       ENTERPRISE,
                       ROLE_CONTEXT,
                       cmf.monitor.hostmonitor.json_parser.SAMPLE)],
    SERVICEMONITOR=\
        [MetricDetails(MGMT_FORMAT,
                       ENTERPRISE,
                       ROLE_CONTEXT,
                       cmf.monitor.servicemonitor.json_parser.SAMPLE)],
    ACTIVITYMONITOR=\
        [MetricDetails(MGMT_FORMAT,
                       ENTERPRISE,
                       ROLE_CONTEXT,
                       cmf.monitor.activitymonitor.json_parser.SAMPLE)],
    EVENTSERVER=\
        [MetricDetails(MGMT_FORMAT,
                       ENTERPRISE,
                       ROLE_CONTEXT,
                       cmf.monitor.eventserver.json_parser.SAMPLE)]
    )

METRIC_NAME_OVERRIDES = \
    dict({# We alias these four capacity related metrics for historical reasons:
          # we always stored these metrics under different metric IDs than those
          # for directory or filesystem capacity, so now we are sort of stuck
          # using them (or we can decide to lose some history). This seems fine.
          'Hadoop:service=,name=NameNodeInfo::Free': 'dfs_capacity_free',
          'Hadoop:service=,name=NameNodeInfo::Used': 'dfs_capacity_used',
          'Hadoop:service=,name=NameNodeInfo::Total': 'dfs_capacity',
          'Hadoop:service=,name=NameNodeInfo::NonDfsUsedSpace': 'dfs_capacity_used_non_hdfs',
          # We alias the following in CDH3 because of the capitalization. This
          # makes it match the capitalization in CDH4.
          'dfs.namenode.SafemodeTime': 'safe_mode_time',
          # This metric was renamed in CDH4, so we alias the CDH3 version to the
          # new name.
          'dfs.namenode.JournalTransactionsBatchedInSync': 'transactions_batched_in_sync',
          # Note that between CDH3 and CDH4 the semantic of corrupt blocks
          # changed, so we map it to different metrics. We also adjust the names
          # to make it more clear what they are tracking. Appropriate aliases
          # have been added in the schema.
          'dfs.FSNamesystem.CorruptBlocks': 'corrupt_replicas',
          'Hadoop:service=,name=FSNamesystem::CorruptBlocks': 'blocks_with_corrupt_replicas',
          # Note the inconsistent capitalization of 'heartBeat'. We alias this
          # metric to the more common capitalization for consistency.
          'dfs.datanode.heartBeats_num_ops': 'heartbeats_num_ops',
          'dfs.datanode.heartBeats_avg_time': 'heartbeats_avg_time',
          # We have to alias this metric since we collect a 'currentLagTxns'
          # metric from both the NameNode and the JournalNode, and we store both
          # under the JournalNode. Without aliasing, we would have collisions.
          'Hadoop:service=,name=Journal-journal::CurrentLagTxns': 'current_lag_txns_journalnode',
          # These metrics are inconsistently named across the JobTracker and the
          # TaskTracker, so we make them consistent by aliasing the JobTracker
          # version. We have added appropiate aliases in the schema.
          'mapred.jobtracker.running_maps': 'maps_running',
          'mapred.jobtracker.running_reduces': 'reduces_running',
          # We rename the following HBase metrics for clarity. RegionServerMetrics
          # is re-using the _num_ops _avg_time infrastructure to track compaction
          # and flush sizes.
          'hbase.regionserver.compactionSize_avg_time': 'compaction_avg_size',
          'hbase.regionserver.flushSize_avg_time': 'flush_avg_size',
          'hbase.regionserver.compactionTime_avg_time': 'compaction_avg_time',
          'hbase.regionserver.flushTime_avg_time': 'flush_avg_time',
          'hbase.regionserver.compactionTime_num_ops': 'compaction_num_ops',
          'hbase.regionserver.flushTime_num_ops': 'flush_num_ops',
          # Similarly, MasterMetrics is doing the same for HLog split stats.
          'hbase.master.splitSize_avg_time': 'split_avg_size',
          'hbase.master.splitTime_avg_time': 'split_avg_time',
          'hbase.master.splitTime_num_ops': 'split_num_ops',
          # Hadoop JVM Metrics
          'jvm.metrics.maxMemoryM': 'jvm_max_memory_mb',
          'jvm.metrics.gcCount': 'jvm_gc_count',
          'Hadoop:service=,name=JvmMetrics::GcCount': 'jvm_gc_count',
          'jvm.metrics.gcTimeMillis': 'jvm_gc_time_ms',
          'Hadoop:service=,name=JvmMetrics::GcTimeMillis': 'jvm_gc_time_ms',
          'jvm.metrics.threadsNew': 'jvm_new_threads',
          'Hadoop:service=,name=JvmMetrics::ThreadsNew': 'jvm_new_threads',
          'jvm.metrics.threadsRunnable': 'jvm_runnable_threads',
          'Hadoop:service=,name=JvmMetrics::ThreadsRunnable': 'jvm_runnable_threads',
          'jvm.metrics.threadsBlocked': 'jvm_blocked_threads',
          'Hadoop:service=,name=JvmMetrics::ThreadsBlocked': 'jvm_blocked_threads',
          'jvm.metrics.threadsTerminated': 'jvm_terminated_threads',
          'Hadoop:service=,name=JvmMetrics::ThreadsTerminated': 'jvm_terminated_threads',
          'jvm.metrics.threadsWaiting': 'jvm_waiting_threads',
          'Hadoop:service=,name=JvmMetrics::ThreadsWaiting': 'jvm_waiting_threads',
          'jvm.metrics.threadsTimedWaiting': 'jvm_timed_waiting_threads',
          'Hadoop:service=,name=JvmMetrics::ThreadsTimedWaiting': 'jvm_timed_waiting_threads',
          'jvm.metrics.memNonHeapCommittedM': 'jvm_non_heap_committed_mb',
          'Hadoop:service=,name=JvmMetrics::MemNonHeapUsedM': 'jvm_non_heap_used_mb',
          'jvm.metrics.memNonHeapUsedM': 'jvm_non_heap_used_mb',
          'Hadoop:service=,name=JvmMetrics::MemNonHeapCommittedM': 'jvm_non_heap_committed_mb',
          'jvm.metrics.memHeapUsedM': 'jvm_heap_used_mb',
          'Hadoop:service=,name=JvmMetrics::MemHeapUsedM': 'jvm_heap_used_mb',
          'jvm.metrics.memHeapCommittedM': 'jvm_heap_committed_mb',
          'Hadoop:service=,name=JvmMetrics::MemHeapCommittedM': 'jvm_heap_committed_mb',
          # RPC System Metrics
          'rpc.metrics.SentBytes': 'rpc_sent_bytes',
          'Hadoop:service=,name=RpcActivityForPort::SentBytes': 'rpc_sent_bytes',
          'rpc.metrics.ReceivedBytes': 'rpc_received_bytes',
          'Hadoop:service=,name=RpcActivityForPort::ReceivedBytes': 'rpc_received_bytes',
          'rpc.metrics.callQueueLen': 'rpc_call_queue_length',
          'Hadoop:service=,name=RpcActivityForPort::CallQueueLength': 'rpc_call_queue_length',
          'rpc.metrics.NumOpenConnections': 'rpc_num_open_connections',
          'Hadoop:service=,name=RpcActivityForPort::NumOpenConnections': 'rpc_num_open_connections',
          # Metrics System Metrics
          'Hadoop:service=,name=MetricsSystem,sub=Stats::DroppedPubAll': 'metrics_dropped_pub_all',
          'Hadoop:service=,name=MetricsSystem,sub=Stats::NumActiveSinks': 'metrics_num_active_sinks',
          'Hadoop:service=,name=MetricsSystem,sub=Stats::NumActiveSources': 'metrics_num_active_sources',
          'Hadoop:service=,name=MetricsSystem,sub=Stats::NumAllSinks': 'metrics_num_all_sinks',
          'Hadoop:service=,name=MetricsSystem,sub=Stats::NumAllSources': 'metrics_num_all_sources',
          'Hadoop:service=,name=MetricsSystem,sub=Stats::PublishAvgTime': 'metrics_publish_avg_time',
          'Hadoop:service=,name=MetricsSystem,sub=Stats::PublishNumOps': 'metrics_publish_num_ops',
          'Hadoop:service=,name=MetricsSystem,sub=Stats::SnapshotAvgTime': 'metrics_snapshot_avg_time',
          'Hadoop:service=,name=MetricsSystem,sub=Stats::SnapshotNumOps': 'metrics_snapshot_num_ops',
          # Event publisher stats
          'com.cloudera.cmf.event.publish:type=EventStorePublisherWithRetryMXBean::SendQueueSize': 'event_publisher_send_queue_size',
          'com.cloudera.cmf.event.publish:type=EventStorePublisherWithRetryMXBean::DropCount': 'event_publisher_drop_count',
          'com.cloudera.cmf.event.publish:type=EventStorePublisherWithRetryMXBean::ExceptionCount': 'event_publisher_exception_count',
          'com.cloudera.cmf.event.publish:type=EventStorePublisherWithRetryMXBean::PushbackCount': 'event_publisher_pushback_count',
          'com.cloudera.cmf.event.publish:type=EventStorePublisherWithRetryMXBean::SentCount': 'event_publisher_sent_count',
          # Hmon Pipeline metrics
          'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=hostStageTSStore::EventsDropped' : 'hmon_host_drops',
          'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=hostStageTSStore::EventsProcessed' : 'hmon_host_processed',
          'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=hostStageTSStore::InputQueueSize' : 'hmon_host_queue_size',
          # Smon Pipeline metrics
          'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=roleStageTSStore::EventsDropped' : 'smon_role_drops',
          'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=roleStageTSStore::EventsProcessed' : 'smon_role_processed',
          'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=roleStageTSStore::InputQueueSize' : 'smon_role_queue_size',
          # Amon Pipeline metrics - activity_tree
          'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=activityTree::EventsDropped' : 'amon_activity_tree_drops',
          'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=activityTree::EventsProcessed' : 'amon_activity_tree_processed',
          'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=activityTree::InputQueueSize' : 'amon_activity_tree_queue_size',
          # Amon Pipeline metrics - activity_monitoring
          'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=activityMonitoring::EventsDropped' : 'amon_activity_monitor_drops',
          'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=activityMonitoring::EventsProcessed' : 'amon_activity_monitor_processed',
          'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=activityMonitoring::InputQueueSize' : 'amon_activity_monitor_queue_size',
          # Amon Pipeline metrics - activity_and_attempt_purger
          'com.cloudera.enterprise:type=PeriodicEnterpriseServiceMXBean,service=ActivityAndAttemptPurgeService::RunCount' : 'amon_purger_service_runs',
          'com.cloudera.enterprise:type=PeriodicEnterpriseServiceMXBean,service=ActivityAndAttemptPurgeService::SlowRunCount' : 'amon_purger_service_slow_runs',
          'com.cloudera.enterprise:type=PeriodicEnterpriseServiceMXBean,service=ActivityAndAttemptPurgeService::RunWithExceptionCount' : 'amon_purger_service_runs_with_exception',
          # EventCatcher ServiceMXBean metrics
          'com.cloudera.cmf.eventcatcher.server:type=EventCatcherServiceMXBean::EventCount' : 'event_count',
          'com.cloudera.cmf.eventcatcher.server:type=EventCatcherServiceMXBean::MaxEventCapacity' : 'event_capacity',
          # EventServer Pipeline metrics
          'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=tagger-writer::EventsDropped' : 'event_writer_drops',
          'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=tagger-writer::EventsProcessed' : 'event_writer_processed',
          'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=tagger-writer::InputQueueSize' : 'event_writer_queue_size'})

IGNORED_METRICS = \
    set([# We ignore the GB version of the capacity metrics.
         'dfs.FSNamesystem.CapacityUsedGB',
         'Hadoop:service=,name=FSNamesystem::CapacityUsedGB',
         'dfs.FSNamesystem.CapacityTotalGB',
         'Hadoop:service=,name=FSNamesystem::CapacityTotalGB',
         'dfs.FSNamesystem.CapacityRemainingGB',
         'Hadoop:service=,name=FSNamesystem::CapacityRemainingGB',
         # This metric is duplicated by the FilesTotal metric in the same bean.
         'Hadoop:service=,name=FSNamesystem::TotalFiles',
         # We consider this metric in our calculation of the HDFS's upgrade
         # status, but we don't collect and store it directly per-se, so we will
         # mark it as ignored.
         'Hadoop:service=,name=NameNodeInfo::UpgradeFinalized',
         # The following are either not appropiate for metrics or are duplicated
         # by metrics we do collect.
         'Hadoop:service=,name=NameNodeInfo::BlockPoolId',
         'Hadoop:service=,name=NameNodeInfo::Version',
         'Hadoop:service=,name=NameNodeInfo::NumberOfMissingBlocks',
         'Hadoop:service=,name=NameNodeInfo::Safemode',
         'Hadoop:service=,name=NameNodeInfo::TotalBlocks',
         'Hadoop:service=,name=NameNodeInfo::TotalFiles',
         'Hadoop:service=,name=NameNodeInfo::ClusterId',
         'Hadoop:service=,name=NameNodeInfo::Threads',
         'Hadoop:service=,name=NameNodeInfo::LiveNodes',
         'Hadoop:service=,name=NameNodeInfo::DecomNodes',
         'Hadoop:service=,name=NameNodeInfo::DeadNodes',
         'Hadoop:service=,name=NameNodeInfo::NameDirStatuses',
         # We currently ignore the entire FSDatasetState bean contents, but we
         # still track it in case something interesting shows up there in the
         # future. All this stuff is either not really a metric or is tracked
         # elsewhere.
         'Hadoop:service=,name=FSDatasetState::Capacity',
         'Hadoop:service=,name=FSDatasetState::StorageInfo',
         'Hadoop:service=,name=FSDatasetState::NumFailedVolumes',
         'Hadoop:service=,name=FSDatasetState::Remaining',
         'Hadoop:service=,name=FSDatasetState::DfsUsed',
         # We currently ignore the entire FSNamesystemState bean contents, but
         # we still track it in case something interesting shows up there in
         # the future. All this stuff is either not really a metric or is
         # tracked elsewhere.         
         'Hadoop:service=,name=FSNamesystemState::FSState',
         'Hadoop:service=,name=FSNamesystemState::CapacityTotal',
         'Hadoop:service=,name=FSNamesystemState::CapacityUsed',
         'Hadoop:service=,name=FSNamesystemState::CapacityRemaining',
         'Hadoop:service=,name=FSNamesystemState::BlocksTotal',
         'Hadoop:service=,name=FSNamesystemState::UnderReplicatedBlocks',
         'Hadoop:service=,name=FSNamesystemState::NumDeadDataNodes',
         'Hadoop:service=,name=FSNamesystemState::NumLiveDataNodes',
         'Hadoop:service=,name=FSNamesystemState::FilesTotal',
         # IPCLoggerChannel metrics from the NameNode that are ignored in the
         # NameNode context and instead written as JournalNode metrics.
         'Hadoop:service=,name=IPCLoggerChannel::QueuedEditsSize',
         'Hadoop:service=,name=IPCLoggerChannel::LagTimeMillis',
         'Hadoop:service=,name=IPCLoggerChannel::CurrentLagTxns',
         # The following are either not appropiate for metrics or are duplicated
         # by metrics we do collect.
         'Hadoop:service=,name=DataNodeInfo::VolumeInfo',
         'Hadoop:service=,name=DataNodeInfo::HttpPort',
         'Hadoop:service=,name=DataNodeInfo::ClusterId',
         'Hadoop:service=,name=DataNodeInfo::RpcPort',
         'Hadoop:service=,name=DataNodeInfo::NamenodeAddresses',
         'Hadoop:service=,name=DataNodeInfo::Version',
         # According to Todd nothing ever called these RPCs, and they have are
         # removed in trunk, so no need to collect them.
         'rpc.detailed-metrics.distributedUpgradeProgress_num_ops',
         'rpc.detailed-metrics.distributedUpgradeProgress_avg_time',
         'Hadoop:service=,name=RpcDetailedActivityForPort::DistributedUpgradeProgressNumOps',
         'Hadoop:service=,name=RpcDetailedActivityForPort::DistributedUpgradeProgressAvgTime',
         'rpc.detailed-metrics.processUpgradeCommand_num_ops',
         'rpc.detailed-metrics.processUpgradeCommand_avg_time',
         'Hadoop:service=,name=RpcDetailedActivityForPort::ProcessUpgradeCommandNumOps',
         'Hadoop:service=,name=RpcDetailedActivityForPort::ProcessUpgradeCommandAvgTime',
         # The following are completed duplicated by RPC metrics, so we ignore them.
         'Hadoop:service=,name=NameNodeActivity::GetLinkTargetOps',
         'Hadoop:service=,name=NameNodeActivity::GetAdditionalDatanodeOps',
         'Hadoop:service=,name=NameNodeActivity::CreateSymlinkOps',
         # This metric tracks such a rare event -- an upgrade once per major
         # CDH version upgrade, that it's not worth the clutter.
         'rpc.detailed-metrics.finalizeUpgrade_num_ops',
         'rpc.detailed-metrics.finalizeUpgrade_avg_time',
         'Hadoop:service=,name=RpcDetailedActivityForPort::FinalizeUpgradeNumOps',
         'Hadoop:service=,name=RpcDetailedActivityForPort::FinalizeUpgradeAvgTime',
         # Both this and hbase.regionserver.compactionTime_num_ops get bumped
         # on every compaction, so we only need one. By fiat we choose to ignore
         # this one. Likewise for flushes.
         'hbase.regionserver.compactionSize_num_ops',
         'hbase.regionserver.flushSize_num_ops',
         # It's a similar story for hbase.master.splitSize_num_ops.
         'hbase.master.splitSize_num_ops',
         # These metrics are related to the call to stop the Master or shutdown
         # the cluster as a whole. They would just be clutter if we showed them,
         # since they will always be zero if we can see metrics.
        'rpc.metrics.stopMaster_num_ops',
        'rpc.metrics.stopMaster_avg_time',
        'rpc.metrics.stopMaster.aboveOneSec._num_ops',
        'rpc.metrics.stopMaster.aboveOneSec._avg_time',
        'rpc.metrics.shutdown_num_ops',
        'rpc.metrics.shutdown_avg_time',
        'rpc.metrics.shutdown.aboveOneSec._num_ops',
        'rpc.metrics.shutdown.aboveOneSec._avg_time',
         # These metrics are related to the call to stop the RegionServer or to
         # fail RegionServer startup. They would just be clutter if we showed
         # them, since they will always be zero if we can see metrics.
         'rpc.metrics.abort_num_ops',
         'rpc.metrics.abort_avg_time',
         'rpc.metrics.abort.aboveOneSec._num_ops',
         'rpc.metrics.abort.aboveOneSec._avg_time',
         'rpc.metrics.stop_num_ops',
         'rpc.metrics.stop_avg_time',
         'rpc.metrics.stop.aboveOneSec._num_ops',
         'rpc.metrics.stop.aboveOneSec._avg_time',
         'rpc.metrics.isStopped_num_ops',
         'rpc.metrics.isStopped_avg_time',
         'rpc.metrics.isStopped.aboveOneSec._num_ops',
         'rpc.metrics.isStopped.aboveOneSec._avg_time',
         'rpc.metrics.checkOOME_num_ops',
         'rpc.metrics.checkOOME_avg_time',
         'rpc.metrics.checkOOME.aboveOneSec._num_ops',
         'rpc.metrics.checkOOME.aboveOneSec._avg_time',
         # The following are internal methods to HRegionServer which don't appear
         # to be interesting enough to expose.
         'rpc.metrics.getFromOnlineRegions_num_ops',
         'rpc.metrics.getFromOnlineRegions_avg_time',
         'rpc.metrics.getFromOnlineRegions.aboveOneSec._num_ops',
         'rpc.metrics.getFromOnlineRegions.aboveOneSec._avg_time',
         'rpc.metrics.addToOnlineRegions_num_ops',
         'rpc.metrics.addToOnlineRegions_avg_time',
         'rpc.metrics.addToOnlineRegions.aboveOneSec._num_ops',
         'rpc.metrics.addToOnlineRegions.aboveOneSec._avg_time',
         'rpc.metrics.removeFromOnlineRegions_num_ops',
         'rpc.metrics.removeFromOnlineRegions_avg_time',
         'rpc.metrics.removeFromOnlineRegions.aboveOneSec._num_ops',
         'rpc.metrics.removeFromOnlineRegions.aboveOneSec._avg_time',
         # The following are not appropriate as metrics.
         'impala-server.ready',
         'impala-server.start-time',
         'simple-scheduler.initialized',
         'statestore.backend-state-map',
         'statestore.live-backends.list',
         # Event Publisher stats that have been ignored historically.
         'com.cloudera.cmf.event.publish:type=EventStorePublisherWithRetryMXBean::SendQueueMaxSize',
         # Hmon Pipeline metrics that have been ignored historically.
         'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=hostStageTSStore::StageName',
         'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=hostStageTSStore::EventsSkipped',
         'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=hostStageTSStore::EventsForwarded',
         'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=hostStageTSStore::InputQueueCapacity',
         # Smon Pipeline metrics that have been ignored historically.
         'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=roleStageTSStore::StageName',
         'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=roleStageTSStore::EventsSkipped',
         'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=roleStageTSStore::EventsForwarded',
         'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=roleStageTSStore::InputQueueCapacity',
         # Amon Pipeline activity tree metrics that have been ignored historically.
         'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=activityTree::StageName',
         'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=activityTree::EventsSkipped',
         'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=activityTree::EventsForwarded',
         'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=activityTree::InputQueueCapacity',
         # Amon Pipeline activity monitor metrics that have been ignored historically.
         'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=activityMonitoring::StageName',
         'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=activityMonitoring::EventsSkipped',
         'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=activityMonitoring::EventsForwarded',
         'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=activityMonitoring::InputQueueCapacity',
         # Amon activity and attempt purge service metrics that have been ignored historically.
         'com.cloudera.enterprise:type=PeriodicEnterpriseServiceMXBean,service=ActivityAndAttemptPurgeService::ServiceName',
         'com.cloudera.enterprise:type=PeriodicEnterpriseServiceMXBean,service=ActivityAndAttemptPurgeService::RunsTotalDuration',
         'com.cloudera.enterprise:type=PeriodicEnterpriseServiceMXBean,service=ActivityAndAttemptPurgeService::LastRunDuration',
         'com.cloudera.enterprise:type=PeriodicEnterpriseServiceMXBean,service=ActivityAndAttemptPurgeService::LastRunStart',
         'com.cloudera.enterprise:type=PeriodicEnterpriseServiceMXBean,service=ActivityAndAttemptPurgeService::LastRunEnd',
         # EventServer Pipeline metrics that have been ignored historically.
         'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=tagger-writer::StageName',
         'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=tagger-writer::EventsSkipped',
         'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=tagger-writer::EventsForwarded',
         'com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=tagger-writer::InputQueueCapacity'])

# We work around CDH-10402 by ignoring particular metrics for the HBase Master
# and RegionServer. See the JIRA for more information.
IGNORED_REGIONSERVER_METRICS = \
    set([# From the HMasterInterface interface.
         'rpc.metrics.isMasterRunning_num_ops',
         'rpc.metrics.isMasterRunning_avg_time',
         'rpc.metrics.createTable_num_ops',
         'rpc.metrics.createTable_avg_time',
         'rpc.metrics.deleteTable_num_ops',
         'rpc.metrics.deleteTable_avg_time',
         'rpc.metrics.addColumn_num_ops',
         'rpc.metrics.addColumn_avg_time',
         'rpc.metrics.modifyColumn_num_ops',
         'rpc.metrics.modifyColumn_avg_time',
         'rpc.metrics.deleteColumn_num_ops',
         'rpc.metrics.deleteColumn_avg_time',
         'rpc.metrics.enableTable_num_ops',
         'rpc.metrics.enableTable_avg_time',
         'rpc.metrics.disableTable_num_ops',
         'rpc.metrics.disableTable_avg_time',
         'rpc.metrics.modifyTable_num_ops',
         'rpc.metrics.modifyTable_avg_time',
         'rpc.metrics.shutdown_num_ops',
         'rpc.metrics.shutdown_avg_time',
         'rpc.metrics.stopMaster_num_ops',
         'rpc.metrics.stopMaster_avg_time',
         'rpc.metrics.getClusterStatus_num_ops',
         'rpc.metrics.getClusterStatus_avg_time',
         'rpc.metrics.move_num_ops',
         'rpc.metrics.move_avg_time',
         'rpc.metrics.assign_num_ops',
         'rpc.metrics.assign_avg_time',
         'rpc.metrics.unassign_num_ops',
         'rpc.metrics.unassign_avg_time',
         'rpc.metrics.offline_num_ops',
         'rpc.metrics.offline_avg_time',
         'rpc.metrics.balance_num_ops',
         'rpc.metrics.balance_avg_time',
         'rpc.metrics.balanceSwitch_num_ops',
         'rpc.metrics.balanceSwitch_avg_time',
         # From the HMasterRegionInterface
         'rpc.metrics.regionServerStartup_num_ops',
         'rpc.metrics.regionServerStartup_avg_time',
         'rpc.metrics.regionServerReport_num_ops',
         'rpc.metrics.regionServerReport_avg_time',
         'rpc.metrics.reportRSFatalError_num_ops',
         'rpc.metrics.reportRSFatalError_avg_time'])
IGNORED_MASTER_METRICS = \
    set([# From the HRegionInterface interface
        'rpc.metrics.getRegionInfo_num_ops',
        'rpc.metrics.getRegionInfo_avg_time',
        'rpc.metrics.getClosestRowBefore_num_ops',
        'rpc.metrics.getClosestRowBefore_avg_time',
        'rpc.metrics.get_num_ops',
        'rpc.metrics.get_avg_time',
        'rpc.metrics.exists_num_ops',
        'rpc.metrics.exists_avg_time',
        'rpc.metrics.put_num_ops',
        'rpc.metrics.put_avg_time',
        'rpc.metrics.delete_num_ops',
        'rpc.metrics.delete_avg_time',
        'rpc.metrics.checkAndPut_num_ops',
        'rpc.metrics.checkAndPut_avg_time',
        'rpc.metrics.checkAndDelete_num_ops',
        'rpc.metrics.checkAndDelete_avg_time',
        'rpc.metrics.incrementColumnValue_num_ops',
        'rpc.metrics.incrementColumnValue_avg_time',
        'rpc.metrics.increment_num_ops',
        'rpc.metrics.increment_avg_time',
        'rpc.metrics.openScanner_num_ops',
        'rpc.metrics.openScanner_avg_time',
        'rpc.metrics.next_num_ops',
        'rpc.metrics.next_avg_time',
        'rpc.metrics.close_num_ops',
        'rpc.metrics.close_avg_time',
        'rpc.metrics.lockRow_num_ops',
        'rpc.metrics.lockRow_avg_time',
        'rpc.metrics.unlockRow_num_ops',
        'rpc.metrics.unlockRow_avg_time',
        'rpc.metrics.getOnlineRegions_num_ops',
        'rpc.metrics.getOnlineRegions_avg_time',
        'rpc.metrics.getHServerInfo_num_ops',
        'rpc.metrics.getHServerInfo_avg_time',
        'rpc.metrics.multi_num_ops',
        'rpc.metrics.multi_avg_time',
        'rpc.metrics.multiPut_num_ops',
        'rpc.metrics.multiPut_avg_time',
        'rpc.metrics.bulkLoadHFile_num_ops',
        'rpc.metrics.bulkLoadHFile_avg_time',
        'rpc.metrics.openRegion_num_ops',
        'rpc.metrics.openRegion_avg_time',
        'rpc.metrics.openRegions_num_ops',
        'rpc.metrics.openRegions_avg_time',
        'rpc.metrics.closeRegion_num_ops',
        'rpc.metrics.closeRegion_avg_time',
        'rpc.metrics.flushRegion_num_ops',
        'rpc.metrics.flushRegion_avg_time',
        'rpc.metrics.splitRegion_num_ops',
        'rpc.metrics.splitRegion_avg_time',
        'rpc.metrics.compactRegion_num_ops',
        'rpc.metrics.compactRegion_avg_time',
        'rpc.metrics.replicateLogEntries_num_ops',
        'rpc.metrics.replicateLogEntries_avg_time',
        'rpc.metrics.bulkLoadHFiles_num_ops',
        'rpc.metrics.bulkLoadHFiles_avg_time',
        'rpc.metrics.getCompactionState_num_ops',
        'rpc.metrics.getCompactionState_avg_time'])

_ENTERPRISE_ROLES = ['ACTIVITYMONITOR', 'SERVICEMONITOR', 'HOSTMONITOR', 
                     'EVENTSERVER', 'REPORTSMANAGER', 'ALERTPUBLISHER',
                     'NAVIGATOR']

_CDH3_SERVICES = ['HDFS', 'MAPREDUCE', 'HBASE', 'ZOOKEEPER', 'FLUME']

_CDH3_ROLES = ['NAMENODE', 'SECONDARYNAMENODE', 'DATANODE',
               'TASKTRACKER', 'JOBTRACKER',
               'MASTER', 'REGIONSERVER', 'HBASETHRIFTSERVER', 'HBASERESTSERVER',
               'SERVER', 
               'AGENT']

_CDH4_ROLES = ['IMPALAD', 'STATESTORE', 'JOURNALNODE', 'HTTPFS', 'FAILOVERCONTROLLER']

_CDH4_SERVICES = ['IMPALA']

def _event_server_metrics():
    return set(['alerts',
                'events_informational',
                'events_important',
                'events_critical'])

EVENT_SERVER_METRICS = dict()
EVENT_SERVER_METRICS[ENTERPRISE] = dict()
EVENT_SERVER_METRICS[ENTERPRISE]['HOST'] = _event_server_metrics()
EVENT_SERVER_METRICS[ENTERPRISE]['MGMT'] = _event_server_metrics()
for role in _ENTERPRISE_ROLES:
    EVENT_SERVER_METRICS[ENTERPRISE][role] = _event_server_metrics()

EVENT_SERVER_METRICS[CDH3] = dict()
for role in _CDH3_ROLES:
    EVENT_SERVER_METRICS[CDH3][role] = _event_server_metrics()
for service in _CDH3_SERVICES:
    EVENT_SERVER_METRICS[CDH3][service] = _event_server_metrics()

EVENT_SERVER_METRICS[CDH4] = copy.deepcopy(EVENT_SERVER_METRICS[CDH3])
for role in _CDH4_ROLES:
    EVENT_SERVER_METRICS[CDH4][role] = _event_server_metrics()
for service in _CDH4_SERVICES:
    EVENT_SERVER_METRICS[CDH4][service] = _event_server_metrics()

def _hmon_role_metrics():
    return set(['host_health',
                'host_subject_status_1'])

HMON_METRICS = dict()
HMON_METRICS[ENTERPRISE] = \
    dict(HOST=\
             set(['scm_health',
                  'kaiser_health',
                  'subject_status_1']),
         CLUSTER=\
             set(['subject_status_1']))

for role in _ENTERPRISE_ROLES:
    HMON_METRICS[ENTERPRISE][role] = _hmon_role_metrics()

HMON_METRICS[CDH3] = dict()

for role in _CDH3_ROLES:
    HMON_METRICS[CDH3][role] = _hmon_role_metrics()

HMON_METRICS[CDH4] = copy.deepcopy(HMON_METRICS[CDH3])

for role in _CDH4_ROLES:
    HMON_METRICS[CDH4][role] = _hmon_role_metrics()

def _smon_htable_metrics():
  return set(['regions',
              'regions_healthy',
              'regions_slow_to_respond',
              'regions_with_errors'])

def _smon_service_metrics():
    return set(['scm_service_state',
                'kaiser_health',
                'subject_status_1'])

def _smon_role_metrics():
    return set(['scm_health',
                'scm_health_reason',
                'scm_role_state',
                'scm_process_state',
                'role_start_time',
                'kaiser_health',
                'subject_status_1'])

SMON_METRICS = dict()
SMON_METRICS[ENTERPRISE] = dict();
SMON_METRICS[ENTERPRISE]['INTERNAL'] = set(['monitor_start_time'])
SMON_METRICS[ENTERPRISE]['MGMT'] = set(['num_running_event_server',
                                        'event_server_kaiser_health',
                                        'event_server_subject_status_1',
                                        'num_running_activity_monitor',
                                        'activity_monitor_kaiser_health',
                                        'amon_subject_status_1',
                                        'num_running_host_monitor',
                                        'host_monitor_kaiser_health',
                                        'hmon_subject_status_1',
                                        'num_running_service_monitor',
                                        'service_monitor_kaiser_health',
                                        'smon_subject_status_1',
                                        'num_running_alert_publisher',
                                        'alert_publisher_kaiser_health',
                                        'alert_publisher_subject_status_1',
                                        'num_running_reports_manager',
                                        'reports_manager_kaiser_health',
                                        'reports_manager_subject_status_1'])
SMON_METRICS[ENTERPRISE]['MGMT'].update(_smon_service_metrics())

for role in _ENTERPRISE_ROLES:
    SMON_METRICS[ENTERPRISE][role] = _smon_role_metrics()

# This is actually not great. See OPSAPS-9817 for more detail.
SMON_METRICS[ENTERPRISE]['DIRECTORY'] = set(['directory_status'])

SMON_METRICS[CDH3] = dict()

SMON_METRICS[CDH3]['HTABLE'] = _smon_htable_metrics()

for service in _CDH3_SERVICES:
    SMON_METRICS[CDH3][service] = _smon_service_metrics()

for role in _CDH3_ROLES:
    SMON_METRICS[CDH3][role] = _smon_role_metrics()

SMON_METRICS[CDH3]['HDFS'].update(set(['dfs_capacity',
                                       'dfs_capacity_used',
                                       'dfs_capacity_used_non_hdfs',
                                       'dfs_num_datanodes',
                                       'dfs_green_datanodes',
                                       'dfs_yellow_datanodes',
                                       'num_running_master_role',
                                       'master_role_kaiser_health',
                                       'master_role_subject_status_1',
                                       'canary_health',
                                       'canary_duration']))
SMON_METRICS[CDH3]['HBASE'].update(set(['master_role_status',
                                        'master_role_kaiser_health',
                                        'longest_running_master_role_start',
                                        'hbase_running_backup_masters',
                                        'hbase_green_running_backup_masters',
                                        'num_running_master_role',
                                        'hbase_num_masters',
                                        'hbase_green_regionservers',
                                        'hbase_yellow_regionservers',
                                        'hbase_num_regionservers',
                                        'hbck_duration',
                                        'canary_duration']))
SMON_METRICS[CDH3]['MAPREDUCE'].update(set(['num_running_master_role',
                                            'master_role_kaiser_health',
                                            'master_role_subject_status_1',
                                            'mr_num_tts',
                                            'mr_green_tts',
                                            'mr_yellow_tts',
                                            'mr_total_data_local_maps',
                                            'mr_total_rack_local_maps',
                                            'mr_total_other_local_maps']))
SMON_METRICS[CDH3]['FLUME'].update(set(['flume_num_agents',
                                        'flume_green_agents',
                                        'flume_yellow_agents']))
SMON_METRICS[CDH3]['ZOOKEEPER'].update(set(['num_running_zk_servers',
                                            'num_running_standalone_zk_servers',
                                            'num_running_follower_zk_servers',
                                            'num_running_leader_zk_servers',
                                            'num_running_observer_zk_servers',
                                            'num_ongoing_zk_leader_elections',
                                            'longest_running_zk_server_role_start',
                                            'canary_health',
                                            'canary_duration',
                                            'zk_num_servers',
                                            'zk_green_servers',
                                            'zk_yellow_servers',
                                            'zk_current_epoch',
                                            'zk_current_xid']))
SMON_METRICS[CDH3]['NAMENODE'].update(set(['hdfs_safe_mode',
                                           'hdfs_checkpoint_period']))
SMON_METRICS[CDH3]['DATANODE'].update(set(['dfs_capacity',
                                           'dfs_capacity_used',
                                           'dfs_capacity_used_non_hdfs',
                                           'slave_master_connectivity']))
SMON_METRICS[CDH3]['MASTER'].update(set(['master_role_status',
                                         'canary_health',
                                         'canary_duration']))
SMON_METRICS[CDH3]['REGIONSERVER'].update(set(['rs_max_store_files',
                                               'slave_master_connectivity']))
SMON_METRICS[CDH3]['TASKTRACKER'].update(set(['tt_blacklisted',
                                              'slave_master_connectivity']))
SMON_METRICS[CDH3]['SERVER'].update(set(['zk_server_avg_req_latency',
                                         'zk_server_data_node_count',
                                         'zk_server_data_size',
                                         'zk_server_data_watch_count',
                                         'zk_server_last_queued_epoch',
                                         'zk_server_last_queued_xid',
                                         'zk_server_max_req_latency',
                                         'zk_server_min_req_latency',
                                         'zk_server_mode',
                                         'zk_server_outstanding_req_count',
                                         'zk_server_packets_received',
                                         'zk_server_packets_sent',
                                         'zk_server_connection_count',
                                         'jvm_heap_committed_mb',
                                         'jvm_heap_used_mb',
                                         'jvm_new_threads',
                                         'jvm_runnable_threads',
                                         'jvm_blocked_threads',
                                         'jvm_waiting_threads',
                                         'jvm_timed_waiting_threads',
                                         'jvm_terminated_threads',
                                         'jvm_total_threads',
                                         'jvm_gc_count',
                                         'jvm_gc_time_ms']))

# We start with CDH4 metrics as a copy of the CDH3 metrics and then will add and
# remove metrics as necessary.
SMON_METRICS[CDH4] = copy.deepcopy(SMON_METRICS[CDH3])

SMON_METRICS[CDH4]['HTABLE'] = _smon_htable_metrics()

for service in _CDH4_SERVICES:
    SMON_METRICS[CDH4][service] = _smon_service_metrics()

for role in _CDH4_ROLES:
    SMON_METRICS[CDH4][role] = _smon_role_metrics()


SMON_METRICS[CDH4]['HDFS'].update(set(['dfs_num_namenodes',
                                       'blocks_with_corrupt_replicas',
                                       'master_role_status',
                                       'master_role_kaiser_health',
                                       'hdfs_standby_namenode_status',
                                       'hdfs_standby_namenode_kaiser_health',
                                       'longest_running_master_role_start',
                                       'pending_deletion_blocks',
                                       'excess_blocks',
                                       'expired_heartbeats',
                                       'total_load',
                                       'pending_replication_blocks',
                                       'scheduled_replication_blocks',
                                       'block_capacity',
                                       'files_total']))
SMON_METRICS[CDH4]['NAMENODE'].update(set(['hdfs_checkpoint_transaction_limit',
                                           'hdfs_upgrade_status',
                                           'journalnodes_in_sync',
                                           'journalnodes_out_of_sync',
                                           'nn_num_directory_failures',
                                           'master_role_status',
                                           'live_datanodes',
                                           'dead_datanodes',
                                           'decomissioning_datanodes']))
SMON_METRICS[CDH4]['JOURNALNODE'] = set(['sync_status',
                                         'queued_edits_size',
                                         'lag_time_millis',
                                         'current_lag_txns_namenode'])
SMON_METRICS[CDH4]['DATANODE'].discard('slave_master_connectivity')
SMON_METRICS[CDH4]['DATANODE'].update(set(['datanode_namenode_connections_good',
                                           'datanode_namenode_connections_bad',
                                           'datanode_namenode_connections_unknown',
                                           'blocks_total']))
SMON_METRICS[CDH4]['IMPALA'] = set(['num_running_master_role',
                                    'master_role_kaiser_health',
                                    'master_role_subject_status_1',
                                    'impala_num_impalads',
                                    'impala_green_impalads',
                                    'impala_yellow_impalads'])
SMON_METRICS[CDH4]['IMPALAD'] = set(['slave_master_connectivity'])
SMON_METRICS[CDH4]['STATESTORE'] = set(['live_backends'])

AMON_METRICS = dict()
AMON_METRICS[CDH3] = dict()
AMON_METRICS[CDH3]['ACTIVITY'] = set(['user',
                                      'group',
                                      'mapper',
                                      'reducer',
                                      'input_dir',
                                      'output_dir',
                                      'queue_name',
                                      'parent_id',
                                      'activity_name',
                                      'activity_type',
                                      'activity_status',
                                      'priority',
                                      'combiner',
                                      'hive_job_id',
                                      'hive_query_string',
                                      'oozie_job_id',
                                      'pig_job_id',
                                      'pig_inp_signatures',
                                      'num_desired_maps',
                                      'num_desired_reduces',
                                      'map_progress',
                                      'reduce_progress',
                                      'submit_time',
                                      'start_time',
                                      'finish_time',
                                      'data_local_maps',
                                      'rack_local_maps',
                                      'other_local_maps',
                                      'slots_millis_maps',
                                      'slots_millis_reduces',
                                      'fallow_slots_millis_maps',
                                      'fallow_slots_millis_reduces',
                                      'total_launched_maps',
                                      'total_launched_reduces',
                                      'maps_failed',
                                      'reduces_failed',
                                      'reduces_running_in_shuffle_phase',
                                      'reduces_running_in_sort_phase',
                                      'reduces_running_in_reduce_phase',
                                      'fairscheduler_demand',
                                      'fairscheduler_fair_share',
                                      'fairscheduler_min_share',
                                      'fairscheduler_running_tasks',
                                      'fairscheduler_weight',
                                      'split_raw_bytes',
                                      'maps_running',
                                      'reduces_running',
                                      'combine_input_records',
                                      'combine_output_records',
                                      'cpu_system',
                                      'cpu_user',
                                      'disk_read',
                                      'disk_write',
                                      'duration',
                                      'hdfs_read',
                                      'hdfs_write',
                                      'id',
                                      'map_input_bytes',
                                      'map_input_records',
                                      'map_output_bytes',
                                      'map_output_records',
                                      'mem_rss',
                                      'mem_virtual',
                                      'reduce_input_groups',
                                      'reduce_input_records',
                                      'reduce_output_records',
                                      'reduce_shuffle_bytes',
                                      'shuffle_finish_time',
                                      'sort_finish_time',
                                      'spilled_records',
                                      'total_cpu_system',
                                      'total_cpu_user'])
AMON_METRICS[CDH3]['ATTEMPT'] = set(['host'])
AMON_METRICS[CDH3]['CLUSTER'] = set(['mem_virtual',
                                     'spilled_records',
                                     'disk_write',
                                     'reduce_shuffle_bytes',
                                     'map_output_records',
                                     'hdfs_write',
                                     'maps_running',
                                     'cpu_user',
                                     'reduces_running',
                                     'split_raw_bytes',
                                     'mem_rss',
                                     'hdfs_read',
                                     'disk_read',
                                     'cpu_system',
                                     'combine_input_records',
                                     'map_slots',
                                     'reduces_running_in_sort_phase',
                                     'reduce_slots',
                                     'reduces_running_in_shuffle_phase',
                                     'map_output_bytes',
                                     'reduce_input_records',
                                     'map_input_bytes',
                                     'reduces_running_in_reduce_phase',
                                     'reduce_output_records',
                                     'map_input_records',
                                     'reduce_input_groups',
                                     'combine_output_records'])

AMON_METRICS[CDH4] = copy.deepcopy(AMON_METRICS[CDH3])

def _java_role_metrics(additional_metrics=[]):
    metrics = set(['jvm_total_threads',
                   'web_metrics_collection_status',
                   'web_metrics_collection_duration'])
    for m in additional_metrics:
        metrics.add(m)
    return metrics

def _non_java_role_metrics(additional_metrics=[]):
    metrics = set(['mem_rss',
                   'mem_virtual',
                   'web_metrics_collection_status',
                   'web_metrics_collection_duration'])
    for m in additional_metrics:
        metrics.add(m)
    return metrics

def _agent_role_metrics():
    return set(['fd_open',
                'fd_max',
                'total_cpu_user',
                'total_cpu_system',
                'unexpected_exits'])

AGENT_METRICS = dict()

AGENT_METRICS[ENTERPRISE] = \
    dict(HOST=\
             set(['load_1',
                  'load_5',
                  'load_15',
                  'cpu_percent',
                  'total_cpu_system',
                  'total_cpu_idle',
                  'total_cpu_iowait',
                  'total_cpu_irq',
                  'total_cpu_user',
                  'total_cpu_nice',
                  'total_cpu_soft_irq',
                  'clock_offset',
                  'agent_physical_memory_used',
                  'agent_fd_open',
                  'agent_fd_max',
                  'agent_cpu_user',
                  'agent_virtual_memory_used',
                  'agent_cpu_system',
                  'agent_up_to_date',
                  'supervisord_fd_max',
                  'supervisord_latency',
                  'supervisord_fd_open',
                  'supervisord_cpu_user',
                  'supervisord_cpu_system',
                  'supervisord_failures',
                  'supervisord_physical_memory_used',
                  'supervisord_virtual_memory_used',
                  'physical_memory_buffers',
                  'physical_memory_cached',
                  'physical_memory_total',
                  'physical_memory_used',
                  'swap_out',
                  'tcp_connection_count_close_wait',
                  'tcp_connection_count_established',
                  'tcp_connection_count_syn_sent',
                  'tcp_connection_count_fin_wait1',
                  'tcp_connection_count_fin_wait2',
                  'tcp_connection_count_syn_recv',
                  'tcp_connection_count_closing',
                  'tcp_connection_count_close',
                  'tcp_connection_count_time_wait',
                  'tcp_connection_count_listen',
                  'tcp_connection_count_last_ack']),
          
         DISK=\
             set(['write_bytes',
                  'write_ios',
                  'read_bytes',
                  'read_ios',
                  'service_time',
                  'await_time',
                  'await_write_time',
                  'await_read_time']),
         
         NETWORK_INTERFACE=\
             set(['speed',
                  'errs_transmit',
                  'colls_transmit',
                  'packets_receive',
                  'packets_transmit',
                  'bytes_receive',
                  'duplex_mode',
                  'errs_receive',
                  'slow_mode',
                  'frame_receive',
                  'drop_receive',
                  'bytes_transmit',
                  'drop_transmit']),

         FILESYSTEM=\
             set(['capacity',
                  'capacity_free',
                  'capacity_used']),

         DIRECTORY=\
             set(['capacity',
                  'capacity_free',
                  'capacity_used']))

for role in _ENTERPRISE_ROLES:
    AGENT_METRICS[ENTERPRISE][role] = _agent_role_metrics()

AGENT_METRICS[ENTERPRISE]['ACTIVITYMONITOR'].update(set(['web_metrics_collection_status',
                                                         'web_metrics_collection_duration']))

AGENT_METRICS[ENTERPRISE]['SERVICEMONITOR'].update(set(['web_metrics_collection_status',
                                                        'web_metrics_collection_duration']))

AGENT_METRICS[ENTERPRISE]['HOSTMONITOR'].update(set(['web_metrics_collection_status',
                                                     'web_metrics_collection_duration']))

AGENT_METRICS[ENTERPRISE]['EVENTSERVER'].update(set(['web_metrics_collection_status',
                                                     'web_metrics_collection_duration']))

AGENT_METRICS[CDH3] = \
    dict(HDFS=\
             set(['blocks_total',
                  'missing_blocks',
                  'under_replicated_blocks',
                  'corrupt_replicas',
                  'pending_deletion_blocks',
                  'total_load',
                  'pending_replication_blocks',
                  'scheduled_replication_blocks',
                  'excess_blocks',
                  'block_capacity',
                  'files_total']),

         MAPREDUCE=\
             set(['map_slots',
                  'reduce_slots',
                  'maps_running',
                  'reduces_running',
                  'trackers_blacklisted',
                  'jobs_completed',
                  'jobs_failed',
                  'jobs_killed',
                  'jobs_preparing',
                  'jobs_running',
                  'jobs_submitted',
                  'maps_failed',
                  'reduces_failed',
                  'waiting_maps',
                  'waiting_reduces']))

for role in _CDH3_ROLES:
    AGENT_METRICS[CDH3][role] = _agent_role_metrics()

AGENT_METRICS[CDH3]['NAMENODE'].update(_java_role_metrics(
        ['nn_ops_read',
         'nn_time_since_last_checkpoint']))

AGENT_METRICS[CDH3]['DATANODE'].update(_java_role_metrics())
AGENT_METRICS[CDH3]['SECONDARYNAMENODE'].update(_java_role_metrics())

AGENT_METRICS[CDH3]['MASTER'].update(_java_role_metrics())
AGENT_METRICS[CDH3]['REGIONSERVER'].update(_java_role_metrics())

AGENT_METRICS[CDH3]['TASKTRACKER'].update(_java_role_metrics())
AGENT_METRICS[CDH3]['JOBTRACKER'].update(_java_role_metrics())
    
AGENT_METRICS[CDH3]['AGENT'].update(set(['web_metrics_collection_status',
                                       'web_metrics_collection_duration']))

AGENT_METRICS[CDH3]['ATTEMPT'] = cmf.monitor.tasktracker.metrics.METRICS

# We start with CDH4 metrics as a copy of the CDH3 metrics and then will add and
# remove metrics as necessary.
AGENT_METRICS[CDH4] = copy.deepcopy(AGENT_METRICS[CDH3])

for role in _CDH4_ROLES:
    AGENT_METRICS[CDH4][role] = _agent_role_metrics()

AGENT_METRICS[CDH4]['JOURNALNODE'].update(_java_role_metrics())
AGENT_METRICS[CDH4]['IMPALAD'].update(_non_java_role_metrics())
AGENT_METRICS[CDH4]['STATESTORE'].update(_non_java_role_metrics())

# CDH4 HDFS service metrics got moved to the HDFS poller, as did the checkpoint
# time or the NameNode.
del AGENT_METRICS[CDH4]['HDFS']
AGENT_METRICS[CDH4]['NAMENODE'].discard('nn_time_since_last_checkpoint')

# TODO: #1. The COMBINER metric never appears to be set to me. I'd like to remove
#           it.
#       #2. Identify which metrics come from which poller for the SMON scale
#           test.
