// (c) Copyright 2011-2012 Cloudera, Inc. All rights reserved.
define([
  'cloudera/navigator/HistorySearch',
  'cloudera/chart/TimeRange',
  'cloudera/Util',
  'underscore'
], function(HistorySearch, TimeRange, Util, _) {
  describe('HistorySearch', function() {
    var $filterElement, $listElement;
    var historySearch, eventSearch;

    // Simulated data from the server.
    var makeTestData = function(length) {
      var results = [];
      _.times(length, function() {
        results.push({
          timestamp: 1348766817260 + (1000000 * results.length)
        });
      });
      return results;
    };

    beforeEach(function() {
      jasmine.Ajax.useMock();
      $filterElement = $('<div></div>').appendTo($('body'));
      $listElement = $('<div></div>').appendTo($('body'));
      historySearch = new HistorySearch(
        $filterElement[0], $listElement[0], 'historyUrl', 'filtersUrl', true);
      eventSearch = historySearch.eventSearch;
    });

    afterEach(function() {
      Util.unsubscribe(historySearch);
      $filterElement.remove();
      $listElement.remove();
    });

    it('has some attributes', function() {
      expect(historySearch.eventSearch).toBeDefined();
      expect(historySearch.subscriptionHandles).toBeDefined();
    });

    it('has a sensible default state upon construction', function() {
      expect(eventSearch.loading()).toBeTruthy();
      expect(historySearch.subscriptionHandles.length).toEqual(2);
    });

    it('takes the right steps when applying filters', function() {
      spyOn(eventSearch, 'deactivateAllFilters');
      var appliedFilters = { test: 'catpants '};
      spyOn(eventSearch, 'getAppliedFilters').andReturn(appliedFilters);
      spyOn(eventSearch, 'loadData');

      historySearch.applyFilters();

      expect(eventSearch.deactivateAllFilters).wasCalled();
      expect(eventSearch.loadData).wasCalledWith(true);
      expect(historySearch.scmUrlParams.filters).
        toEqual(appliedFilters);
    });
    
    it('reloads the data when the time control is updated', function() {
      spyOn(eventSearch, 'loadData');
    
      $.publish('timeSelectionChanged', [new TimeRange(new Date(), new Date())]);
      
      expect(eventSearch.loadData).wasCalledWith(true);
    });

    it('ignores time changes when they are because of auto update', function() {
      spyOn(eventSearch, 'loadData');
      var range = new TimeRange(new Date(), new Date());
      historySearch.onTimeSelectionChanged(range, true, true);
      expect(eventSearch.loadData).wasNotCalled();
    });

    it('knows how to export events as downloadable CSV', function() {
      spyOn(Util, 'setWindowLocation');
      historySearch.exportEvents();
      expect(Util.setWindowLocation).wasCalled();
      var url = Util.setWindowLocation.mostRecentCall.args[0];
      expect(url).toEqual('historyUrl?offset=0&limit=-1&format=CSV&attachment=true');

      historySearch.scmUrlParams.offset = 42;
      Util.setWindowLocation.reset();
      historySearch.exportEvents();
      expect(Util.setWindowLocation).wasCalled();
      url = Util.setWindowLocation.mostRecentCall.args[0];
      expect(url).toEqual('historyUrl?offset=42&limit=-1&format=CSV&attachment=true');
    });

    it('will fire its onTimeSelectionChanged method if it missed the initial' +
        'time selection changed event', function() {
      spyOn(historySearch, 'onTimeSelectionChanged');
      // Blank out any cached values. We expect it not to update itself.
      window._currentTimeSelection = null;
      historySearch.checkInitialTimeRange();
      expect(historySearch.onTimeSelectionChanged).wasNotCalled();

      window._currentTimeSelection = {
        timeRange: 'timeRange',
        isCurrentMode: 'isCurrentMode',
        isAutoUpdate: 'isAutoUpdate'
      };
      historySearch.checkInitialTimeRange();
      expect(historySearch.onTimeSelectionChanged).wasCalledWith(
        'timeRange', 'isCurrentMode', 'isAutoUpdate');
    });

    describe('onRetrieveHistorySuccess callback', function() {
      var mockEventSearch = {
        length: jasmine.createSpy(),
        append: jasmine.createSpy(),
        prepend: jasmine.createSpy(),
        getHead: jasmine.createSpy(),
        getTail: jasmine.createSpy()
      }, mockData;

      var successHandler;

      var makeSuccessHandler = function() {
        successHandler = historySearch.makeOnRetrieveHistorySuccess(false, mockEventSearch);
      };

      beforeEach(function() {
        mockData = [1, 2, 3];
        mockEventSearch.length.andReturn(0);
        makeSuccessHandler();
      });

      it('does nothing if another request has come in before it', function() {
        // Increments the internal state so that a call to the callback
        // won't apply the data.
        historySearch.applyFilters();
        successHandler(mockData);
        expect(mockEventSearch.append).wasNotCalled();
      });

      it('does nothing if no data came back', function() {
        successHandler(null);
        expect(mockEventSearch.append).wasNotCalled();
      });

      it('does nothing if data has length 0', function() {
        successHandler([]);
        expect(mockEventSearch.append).wasNotCalled();
      });

      it('does nothing if event lengths do not match', function() {
        mockEventSearch.length.andReturn(5);
        successHandler(mockData);
        expect(mockEventSearch.append).wasNotCalled();
      });

      it('appends events if there were none to begin with', function() {
        var testData = makeTestData(5);
        successHandler(testData);
        expect(mockEventSearch.append).wasCalled();
        expect(mockEventSearch.getTail).wasNotCalled();
      });

      it('appends data if response from server is older than current oldest', function() {
        mockEventSearch.length.andReturn(5);
        // Set initial length.
        makeSuccessHandler();
        var testData = makeTestData(3);
        mockEventSearch.getTail.andReturn({
          timestamp: testData[2].timestamp + 1000000
        });
        successHandler(testData);
        expect(mockEventSearch.append).wasCalled();
      });

      // This test is more about coverage than it is about testing
      // the underlying logic. So, crummy test. It's true purpose is to
      // provide a scaffold to let me know if I broke things. As such,
      // testing a lot of permutations of data getting prepended is not
      // the immediate goal.
      it('prepends data if newer and found valid prepend range', function() {
        mockEventSearch.length.andReturn(5);
        // Set initial length.
        makeSuccessHandler();
        var testData = makeTestData(3);
        mockEventSearch.getTail.andReturn({
          timestamp: testData[0].timestamp - 50
        });
        mockEventSearch.getHead.andReturn({
          timestamp: testData[2].timestamp + 50
        });
        spyOn(Util, 'findRange').andReturn({
          start: 0,
          end: testData.length - 1
        });
        successHandler(testData);
        expect(mockEventSearch.getTail).wasCalled();
        expect(mockEventSearch.getHead).wasCalled();
        expect(mockEventSearch.prepend).wasCalled();
      });

      it('if called a second time, will set the offsets correctly', function() {
        mockEventSearch.length.andReturn(50);
        expect(historySearch.scmUrlParams.offset).toEqual(0);
        historySearch.makeOnRetrieveHistorySuccess(false, mockEventSearch);
        expect(historySearch.scmUrlParams.offset).toEqual(50);
      });
    });

    describe('dataSource callback', function() {
      var successCallback, completeCallback, mockEventSearch;

      beforeEach(function() {
        mockEventSearch = {
          clear: jasmine.createSpy()
        };
        jasmine.Ajax.useMock();
        successCallback = jasmine.createSpy();
        completeCallback = jasmine.createSpy();
        spyOn(historySearch, 'makeOnRetrieveHistorySuccess').andReturn(successCallback);
      });

      it('clears when asked', function() {
        historySearch.dataSource(mockEventSearch, true, completeCallback);
        expect(mockEventSearch.clear).wasCalled();
      });

      it('calls success callback then complete callback with no error on good response from server', function() {
        historySearch.dataSource(mockEventSearch, false, completeCallback);
        var request = mostRecentAjaxRequest();
        request.response({
          status: 200,
          responseText: JSON.stringify({ stuff: 'catpants' })
        });
        expect(successCallback).wasCalled();
        // Getting called with nothing defined for err; success!
        expect(completeCallback).wasCalledWith(undefined);
      });

      it('calls complete callback with error on bad response from server', function() {
        historySearch.dataSource(mockEventSearch, false, completeCallback);
        var request = mostRecentAjaxRequest();
        // Respond with badly formed JSON to validate the error is
        // passed to the complete callback.
        request.response({
          status: 200,
          responseText: 'stuff!'
        });
        expect(successCallback).wasNotCalled();
        expect(completeCallback).wasCalled();
        expect(completeCallback.mostRecentCall.args.length).toEqual(1);
        expect(completeCallback.mostRecentCall.args[0]).toBeTruthy();
      });
    });

    describe('free edition functionality', function () {
      var $freeFilterElement,
          $freeListElement;

      beforeEach(function() {
        $freeFilterElement = $('<div></div>').appendTo($('body'));
        $freeListElement = $('<div></div>').appendTo($('body'));
      });

      afterEach(function() {
        $freeFilterElement.remove();
        $freeListElement.remove();
      });

      it('should not subscribe to the "timecontrol" event', function () {
        spyOn($, 'subscribe');

        var historySearch = new HistorySearch(
            $freeFilterElement[0], $freeListElement[0], 'historyUrl', 'filtersUrl', false);

        expect($.subscribe).not.toHaveBeenCalledWith("timeSelectionChanged", historySearch.onTimeSelectionChanged);
      });
    });
  });
});
